#include <irrlicht.h>
#include <windows.h>
#include <iostream>
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

#pragma comment(lib, "Irrlicht.lib")

class MyEventReceiver : public IEventReceiver
{
public:
	
	virtual bool OnEvent(const SEvent& event)
	{
		// Remember whether each key is down or up
		if (event.EventType == irr::EET_KEY_INPUT_EVENT)
			KeyIsDown[event.KeyInput.Key] = event.KeyInput.PressedDown;

		return false;
	}

	// This is used to check whether a key is being held down
	virtual bool IsKeyDown(EKEY_CODE keyCode) const
	{
		return KeyIsDown[keyCode];
	}

	MyEventReceiver()
	{
		for (u32 i=0; i<KEY_KEY_CODES_COUNT; ++i)
			KeyIsDown[i] = false;
	}

private:
	//array to store the current state of each key
	bool KeyIsDown[KEY_KEY_CODES_COUNT];
};


int main()
{ video::E_DRIVER_TYPE driverType = video::EDT_DIRECT3D9;

	printf("Please select the driver you want for this example:\n"\
		" (a) Direct3D 9.0c\n (b) Direct3D 8.1\n (c) OpenGL 1.5\n"\
		" (d) Software Renderer\n (e) Burning's Software Renderer\n"\
		" (f) NullDevice\n (otherKey) exit\n\n");

	char i;
	std::cin >> i;

	switch(i)
	{
		case 'a': driverType = video::EDT_DIRECT3D9;break;
		case 'b': driverType = video::EDT_DIRECT3D8;break;
		case 'c': driverType = video::EDT_OPENGL;   break;
		case 'd': driverType = video::EDT_SOFTWARE; break;
		case 'e': driverType = video::EDT_BURNINGSVIDEO;break;
		case 'f': driverType = video::EDT_NULL;     break;
		default: return 0;
	}	

	// create device
	MyEventReceiver receiver;

	IrrlichtDevice* device = createDevice( driverType, core::dimension2d<s32>(640, 480),
		16, false, false, false, &receiver);

	if (device == 0)
		return 1; // could not create selected driver.


	video::IVideoDriver* driver = device->getVideoDriver();
	scene::ISceneManager* smgr = device->getSceneManager();
	
	
	//create skybox
    ISceneNode* SkyBox = 0;
    SkyBox = smgr->addSkyBoxSceneNode(
		driver->getTexture("../../media/textures/skybox/default_skyboxup.jpg"),
		driver->getTexture("../../media/textures/skybox/default_skyboxdn.jpg"),
		driver->getTexture("../../media/textures/skybox/default_skybox1.jpg"), //swap images
		driver->getTexture("../../media/textures/skybox/default_skybox3.jpg"), //for irrlicht 1.4
		driver->getTexture("../../media/textures/skybox/default_skybox2.jpg"),
		driver->getTexture("../../media/textures/skybox/default_skybox0.jpg")); 

    //load .irr file created with IrrEdit
    smgr->loadScene("../../media/terrain.irr");

    //create car mesh (2 models:auto1 and auto 2)
    //IMesh* mesh = smgr->getMesh("../../media/meshes/auto1.obj");
    IMesh* mesh = smgr->getMesh("../../media/meshes/auto2.obj");
	IMeshSceneNode* node = smgr->addMeshSceneNode( mesh );
		if (node)
	{
        node->setScale(vector3df(0.1, 0.1, 0.1));
        //node->setPosition(core::vector3df(2,0,0));
	    node->setPosition(core::vector3df(2,11,-30));
		node->setMaterialTexture(0, driver->getTexture("../../media/meshes/auto_textur.jpg"));
		node->setMaterialFlag(video::EMF_LIGHTING, false);
		
	}
	
	
SKeyMap keyMap[8];
                 keyMap[0].Action = EKA_MOVE_FORWARD;
                 keyMap[0].KeyCode = KEY_UP;
                 

                 keyMap[1].Action = EKA_MOVE_BACKWARD;
                 keyMap[1].KeyCode = KEY_DOWN;
                 

                 keyMap[2].Action = EKA_STRAFE_LEFT;
                 keyMap[2].KeyCode = KEY_LEFT;
                 

                 keyMap[3].Action = EKA_STRAFE_RIGHT;
                 keyMap[3].KeyCode = KEY_RIGHT;
    
    //add camera             
    smgr->addCameraSceneNodeFPS();
    scene::ICameraSceneNode * cam = smgr->addCameraSceneNodeFPS(0,100.0f,100.0f, -1, keyMap, 8, true, 10);
    cam->setPosition(core::vector3df(10,5,-15));
    device->getCursorControl()->setVisible(false);
 
	

	int lastFPS = -1;
	
	while(device->run())
	{
		// Check if key W,key S,key A or Key D is being held down, and move the node
	
		if(receiver.IsKeyDown(irr::KEY_KEY_W))
		{
			core::vector3df v = node->getPosition();
			v.X += 0.1f;
			node->setPosition(v);
		}
		else if(receiver.IsKeyDown(irr::KEY_KEY_S))
		{
			core::vector3df v = node->getPosition();
			v.X -= 0.1f;
			node->setPosition(v);
		}
		else if(receiver.IsKeyDown(irr::KEY_KEY_A))
		{
			core::vector3df v = node->getPosition();
			v.Z += 0.1f;
			node->setPosition(v);
		}
		else if(receiver.IsKeyDown(irr::KEY_KEY_D))
		{
			core::vector3df v = node->getPosition();
			v.Z -= 0.1f;
			node->setPosition(v);
		}
	
        // and draw everything.
        driver->beginScene(true, true, video::SColor(0,200,200,200));
		smgr->drawAll();
		driver->endScene();

		int fps = driver->getFPS();

		if (lastFPS != fps)
		{
		  core::stringw str = L"Landscape with car, trees and houses ";
		  str += driver->getName();
		  str += "] FPS:";
		  str += fps;

		  device->setWindowCaption(str.c_str());
		  lastFPS = fps;
		}

	}

	device->drop();
	
	return 0;
}

